from Acquisition import Implicit
from Persistence import Persistent
from AccessControl.Role import RoleManager
from StructuredText import Basic, Document, HTML, DocBookChapter
from OFS.SimpleItem import Item
from OFS import History
from Globals import HTMLFile, MessageDialog

default_text = """\
This is just some

  Meaningless

  StructuredText
"""

def manage_addSTXDocument(self, id, title='', doc=default_text, REQUEST=None):
    """ add an STXDocument """

    id=str(id)
    title=str(title)

    d=STXDocument(id, title, doc)
    self._setObject(id, d)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
    
manage_addSTXDocumentForm = HTMLFile('addSTXDocument', globals())

class STXDocument(Implicit, Item, Persistent, History.Historical):

    """ A Document object that wraps STXNG Document objects """

    meta_type = 'STX Document'

    manage_options=(
    (
        {'label':'Edit', 'action':'manage_editForm',
         'help':('STXDocument','STX-Document_Edit.stx')},
        {'label':'Upload', 'action':'manage_uploadForm',
         'help':('STXDocument','STX-Document_Upload.stx')},
        {'label':'View', 'action':'',
         'help':('STXDocument','STX-Document_View.stx')},
        )
    +History.Historical.manage_options
    +Item.manage_options
    )
                   
    __ac_permissions__=(
        ('View management screens',
         ('manage_editForm', 'manage', 'manage_main', 'manage_upload',
          'manage_uploadForm', 'document_src', 'PrincipiaSearchSource')),

        ('Change STX Documents',
         ('manage_edit', 'manage_upload', 'PUT')),

        ('View',
         ('get_size', 'index_html', '')),

        ('FTP access',
         ('manage_FTPstat','manage_FTPget','manage_FTPlist')),

        ('Output HTML',
         ('outputHTML',)),
        ('Output DocBook',
         ('outputDocBook',)),
        ('Output Raw STX',
         ('outputRawSTX',)),
        )


    def __init__(self, id, title='', doc='', REQUEST=None):
        self.id = id
        self.manage_edit(title, doc, None)
        
    def manage_edit(self, title='', doc='', REQUEST=None):
        """ at least one doc string gets me every time. """
        self.title=title
        self.document = Document(Basic(str(doc)))
        self._doc = doc

        if REQUEST is not None:
            return self.manage_main(self, REQUEST)

    def manage_upload(self,file='', REQUEST=None):
        """Replace the contents of the document with the text in file."""
        if type(file) is not type(''): file=file.read()
        self.manage_edit(self.title, file)
        
        if REQUEST: return MessageDialog(
                    title  ='Success!',
                    message='Your changes have been saved',
                    action ='manage_main')

        
    manage_uploadForm=HTMLFile('stxDocumentUpload', globals())
    manage_editForm=HTMLFile('stxDocumentEdit', globals())
    manage_main=manage_editForm
    manage_editForm._setName("manage_editForm")

    def index_html(self, REQUEST=None):
        """ eenie meenie minee mo let's do HTML """
        return self.outputHTML(REQUEST)

    def get_size(self):
        """ return size """
        return len(self._doc)

    getSize=get_size

    def document_src(self, REQUEST=None):
        """ return the source of the document """
        return self._doc

    PrincipiaSearchSource=document_src

    def outputHTML(self, REQUEST=None):
        """ output the document in HTML """
        return HTML(self.document)

    def outputDocBook(self, REQUEST=None):
        """ output the document in DocBook """
        return DocBookChapter(self.document)

    def outputRawSTX(self, REQUEST=None, RESPONSE=None):
        """ different than document_src so theres another permission """
        if RESPONSE is not None:
            RESPONSE.setHeader('content-type', 'text/plain')
        return self._doc
    
    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests."""
        self.dav__init(REQUEST, RESPONSE)
        body=REQUEST.get('BODY', '')
        self.manage_edit(self.title, body)
        RESPONSE.setStatus(204)
        return RESPONSE

    def manage_FTPget(self):
        "Get source for FTP download"
        return self.document_src()

    def manage_historyCompare(self, rev1, rev2, REQUEST,
                              historyComparisonResults=''):
        return STXDocument.inheritedAttribute('manage_historyCompare')(
            self, rev1, rev2, REQUEST,
            historyComparisonResults=History.html_diff(
                rev1.document_src(), rev2.document_src()
                ))


## Globals.default__class_init__(STXDocument)







